@@ -217,6 +217,8 @@ atk_bridge_init (gint *argc, gchar **argv[])
       g_error ("Could not initialize Bonobo");
     }
 
+  bonobo_activate ();
+
   /* Create the accessible application server object */
   this_app = spi_application_new (atk_get_root ());
   /*
@@ -275,8 +277,6 @@ spi_atk_bridge_do_registration (void)
       return FALSE;
     }
 
-  bonobo_activate ();
-
   /* Create the accessible application server object */
   if (this_app == NULL)
     this_app = spi_application_new (atk_get_root ());
@@ -507,7 +507,12 @@ spi_atk_register_event_listeners (void)
   AtkObject *bo = atk_no_op_object_new (ao);
 
 
-  if (atk_listeners_registered) return;
+  if (atk_listeners_registered) 
+    {
+      g_object_unref (G_OBJECT (bo));
+      g_object_unref (ao);
+      return;
+    }
 
   atk_listeners_registered = TRUE;
 
@@ -593,8 +598,6 @@ deregister_application (BonoboObject *app)
 
   device_event_controller = bonobo_object_release_unref (device_event_controller, &ev);
   registry = bonobo_object_release_unref (registry, &ev);
-  
-  app = bonobo_object_unref (app);
 }
 
 static void
@@ -740,7 +743,6 @@ spi_atk_emit_eventv (const GObject         *gobject,
   AtkObject          *aobject;
   SpiAccessible      *source = NULL;
   Accessibility_Registry registry;
-  const gchar *name;
 #ifdef SPI_BRIDGE_DEBUG
   CORBA_string s = NULL;
 #endif
@@ -768,7 +770,6 @@ spi_atk_emit_eventv (const GObject         *gobject,
       va_end (args);
       return;
     }
-  name = atk_object_get_name (aobject);
   e.type = g_strdup_vprintf (format, args);
   if (source) e.source = BONOBO_OBJREF (source);
   e.detail1 = detail1;
@@ -1071,8 +1072,10 @@ spi_atk_bridge_key_listener (AtkKeyEventStruct *event, gpointer data)
 
   spi_init_keystroke_from_atk_key_event (&key_event, event);
 
+  bridge_threads_leave ();
   result = Accessibility_DeviceEventController_notifyListenersSync (
 	  spi_atk_bridget_get_dec (), &key_event, &ev);
+  bridge_threads_enter ();
 
   if (key_event.event_string) CORBA_free (key_event.event_string);
 
@@ -1153,19 +1156,28 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
   else if ((signal_query.signal_id == atk_signal_children_changed) && gobject)
     {
       detail1 = g_value_get_uint (param_values + 1);
-      ao = atk_object_ref_accessible_child (ATK_OBJECT (gobject), 
-			        	    detail1);
-      if (ao) 
+      gpointer child = g_value_get_pointer (param_values + 2);
+      if (ATK_IS_OBJECT (child))
+        {
+          ao = ATK_OBJECT (child);
+          g_object_ref (ao);
+        }
+      else
+        {
+          ao = atk_object_ref_accessible_child (ATK_OBJECT (gobject), 
+                                                detail1);
+        }
+      if (ao)
         {
           s_ao = spi_accessible_new (ao);
           c_obj = BONOBO_OBJREF (s_ao);
           spi_atk_bridge_init_object (&any, ATK_OBJECT (gobject), &c_obj);
-	  g_object_unref (ao);
-	}
+          g_object_unref (ao);
+        }
       else
-	{
-	  spi_atk_bridge_init_nil (&any, ATK_OBJECT (gobject));
-	}
+        {
+          spi_atk_bridge_init_nil (&any, ATK_OBJECT (gobject));
+        }
     }
   else
     {
@@ -1293,6 +1305,10 @@ spi_atk_tidy_windows (void)
   gint i;
 
   root = atk_get_root ();
+
+  if (!root)
+    return;
+
   n_children = atk_object_get_n_accessible_children (root);
   for (i = 0; i < n_children; i++)
     {
